Attribute VB_Name = "TransPbmp"
Attribute VB_Description = "Transparent Bitmap BLT - TransBmpH"
'This is a modification of the MS KB article code for
'Transparent bitmaps. This version allows the destination
'to be any object that supports the Hwnd property.

'****The Windows operating system may change the value of the Hwnd for a
'window, so the Hwnd of the destination should be freshly retreived
'each time TransBmpH is called. (See VB help on using the Hwnd property.)


Option Explicit

Private Type BITMAP
    bmType As Long
    bmWidth As Long
    bmHeight As Long
    bmWidthBytes As Long
    bmPlanes As Integer
    bmBitsPixel As Integer
    bmBits As Long
End Type

Private Declare Function BitBlt Lib "gdi32" (ByVal srchDC As Long, ByVal srcX As Long, ByVal srcY As Long, ByVal srcW As Long, ByVal srcH As Long, ByVal desthDC As Long, ByVal destX As Long, ByVal destY As Long, ByVal op As Long) As Long
Private Declare Function SetBkColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long
Private Declare Function ReleaseDC Lib "user32" (ByVal hwnd As Long, ByVal hdc As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function CreateBitmap Lib "gdi32" (ByVal nWidth As Long, ByVal nHeight As Long, ByVal nPlanes As Long, ByVal nBitCount As Long, lpBits As Any) As Long
Private Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function GetObj Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, bmp As Any) As Long

Private Const SRCCOPY = &HCC0020
Private Const SRCAND = &H8800C6
Private Const SRCPAINT = &HEE0086
Private Const NOTSRCCOPY = &H330008

Public Sub TransBmpH(destHwnd As Long, ByVal srcBmp As Long, _
ByVal destX As Long, ByVal destY As Long, ByVal TransColor As Long)
'destHwnd - Hwnd of destination object(any).
'srcBmp - .Picture property of PicureBox, Form, or Image control
'   OR .Handle property of an OLE Picture Object or StdPicture
'destX,destY - target relative (always PIXELS, regardless of dest ScaleMode)
'TransColor - Color to become transparent
'===================================================================
'ADDITIONAL NOTES:
'The source (srcBmp) does NOT have to be visible.
'(srcBmp) can be .Picture property of PicureBox, Form, or Image control
'**OR** the Handle property of an OLE Picture or StdPicture object.
'   With the Handle of a an OLE Picture/StdPicture, the bitmap can be loaded
'   from a RES file without tying up picturebox or image object resources.
'   To use OLE Picutre Objects, a reference to
'   "Standard OLE Types" must be added to the project.
'(dest) can be ANY object for which you can obtain a Hwnd. This would
'include VB objects that support the Hwnd property or any window for
'that matter.

'If the destination is a Form or PictureBox and its AutoRedraw
'property is True, then a Refresh of the destination may be
'necessary for the result to become visible.

'****The Windows operating system may change the value of the Hwnd for a
'window, so the Hwnd of the destination should be freshly retreived
'each time TransBmpH is called. (See VB help on using the Hwnd property.)

    Dim srcDC As Long  'source bitmap (color)
    Dim saveDC As Long 'backup copy of source bitmap
    Dim maskDC As Long 'mask bitmap (monochrome)
    Dim invDC As Long  'inverse of mask bitmap (monochrome)
    Dim resultDC As Long 'combination of source bitmap & background
    Dim bmp As BITMAP 'description of the source bitmap
    Dim hResultBmp As Long 'Bitmap combination of source & background
    Dim hSaveBmp As Long 'Bitmap stores backup copy of source bitmap
    Dim hMaskBmp As Long 'Bitmap stores mask (monochrome)
    Dim hInvBmp As Long  'Bitmap holds inverse of mask (monochrome)
    Dim hPrevBmp As Long 'Bitmap holds previous bitmap selected in DC
    Dim hSrcPrevBmp As Long  'Holds previous bitmap in source DC
    Dim hSavePrevBmp As Long 'Holds previous bitmap in saved DC
    Dim hDestPrevBmp As Long 'Holds previous bitmap in destination DC
    Dim hMaskPrevBmp As Long 'Holds previous bitmap in the mask DC
    Dim hInvPrevBmp As Long 'Holds previous bitmap in inverted mask DC
    Dim OrigColor As Long 'Holds original background color from source DC
    Dim Success As Long 'Stores result of call to Windows API
    Dim DestDC As Long 'DC of final destination
    
    DestDC = GetDC(destHwnd)
    If DestDC <> 0& Then 'if valid
        'Retrieve bitmap to get width (bmp.bmWidth) & height (bmp.bmHeight)
        Success = GetObj(srcBmp, Len(bmp), bmp)
        srcDC = CreateCompatibleDC(DestDC) 'Create DCs to hold stages
        saveDC = CreateCompatibleDC(DestDC) '""
        maskDC = CreateCompatibleDC(DestDC) '""
        invDC = CreateCompatibleDC(DestDC) '""
        resultDC = CreateCompatibleDC(DestDC) '""
        'Create monochrome bitmaps for the mask-related bitmaps:
        hMaskBmp = CreateBitmap(bmp.bmWidth, bmp.bmHeight, 1, 1, ByVal 0&)
        hInvBmp = CreateBitmap(bmp.bmWidth, bmp.bmHeight, 1, 1, ByVal 0&)
        'Create color bitmaps for final result & stored copy of source
        hResultBmp = CreateCompatibleBitmap(DestDC, bmp.bmWidth, bmp.bmHeight) 'tje
        hSaveBmp = CreateCompatibleBitmap(DestDC, bmp.bmWidth, bmp.bmHeight) 'tje
        hSrcPrevBmp = SelectObject(srcDC, srcBmp)     'Select bitmap in DC
        hSavePrevBmp = SelectObject(saveDC, hSaveBmp) 'Select bitmap in DC
        hMaskPrevBmp = SelectObject(maskDC, hMaskBmp) 'Select bitmap in DC
        hInvPrevBmp = SelectObject(invDC, hInvBmp)    'Select bitmap in DC
        hDestPrevBmp = SelectObject(resultDC, hResultBmp) 'Select bitmap
        'Make backup of source bitmap to restore later
        Success = BitBlt(saveDC, 0, 0, bmp.bmWidth, bmp.bmHeight, srcDC, 0, 0, SRCCOPY)
        'Create mask: set background color of source to transparent color.
        OrigColor = SetBkColor(srcDC, TransColor)
        Success = BitBlt(maskDC, 0, 0, bmp.bmWidth, bmp.bmHeight, srcDC, 0, 0, SRCCOPY)
        TransColor = SetBkColor(srcDC, OrigColor)
        'Create inverse of mask to AND w/ source & combine w/ background.
        Success = BitBlt(invDC, 0, 0, bmp.bmWidth, bmp.bmHeight, maskDC, 0, 0, NOTSRCCOPY)
        'Copy background bitmap to result & create final transparent bitmap
        Success = BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight, GetDC(destHwnd), destX, destY, SRCCOPY) 'tje
        'AND mask bitmap w/ result DC to punch hole in the background by
        'painting black area for non-transparent portion of source bitmap.
        Success = BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight, maskDC, 0, 0, SRCAND)
        'AND inverse mask w/ source bitmap to turn off bits associated
        'with transparent area of source bitmap by making it black.
        Success = BitBlt(srcDC, 0, 0, bmp.bmWidth, bmp.bmHeight, invDC, 0, 0, SRCAND)
        'XOR result w/ source bitmap to make background show through.
        Success = BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight, srcDC, 0, 0, SRCPAINT)
        Success = BitBlt(DestDC, destX, destY, bmp.bmWidth, bmp.bmHeight, resultDC, 0, 0, SRCCOPY) 'Display transparent bitmap on backgrnd
        Success = BitBlt(srcDC, 0, 0, bmp.bmWidth, bmp.bmHeight, saveDC, 0, 0, SRCCOPY) 'Restore backup of bitmap.
        hPrevBmp = SelectObject(srcDC, hSrcPrevBmp) 'Select orig object
        hPrevBmp = SelectObject(saveDC, hSavePrevBmp) 'Select orig object
        hPrevBmp = SelectObject(resultDC, hDestPrevBmp) 'Select orig object
        hPrevBmp = SelectObject(maskDC, hMaskPrevBmp) 'Select orig object
        hPrevBmp = SelectObject(invDC, hInvPrevBmp) 'Select orig object
        Success = DeleteObject(hSaveBmp)   'Deallocate system resources.
        Success = DeleteObject(hMaskBmp)   'Deallocate system resources.
        Success = DeleteObject(hInvBmp)    'Deallocate system resources.
        Success = DeleteObject(hResultBmp) 'Deallocate system resources.
        Success = DeleteDC(srcDC)          'Deallocate system resources.
        Success = DeleteDC(saveDC)         'Deallocate system resources.
        Success = DeleteDC(invDC)          'Deallocate system resources.
        Success = DeleteDC(maskDC)         'Deallocate system resources.
        Success = DeleteDC(resultDC)       'Deallocate system resources.
        Call ReleaseDC(destHwnd, DestDC)
    End If
End Sub

